﻿using ComPDFKitViewer.PdfViewer;
using PDF_Office.EventAggregators;
using PDF_Office.Model;
using PDF_Office.Model.EditTools.Background;
using PDF_Office.Model.EditTools.Watermark;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

namespace PDF_Office.ViewModels.EditTools.Watermark
{
    public class WatermarkCreateBaseContentViewModel : BindableBase, INavigationAware
    {
        IEventAggregator eventAggregator;
        IRegionManager watermarkCreateRegion;

        private CPDFViewer PDFViewer;

        public WatermarkInfo WatermarkInfo;

        public EnumTextOrFile CurrentTemplateListMod;

        private string watermarkCreateRegionName;

        public string WatermarkCreateRegionName
        {
            get => watermarkCreateRegionName;
            set => SetProperty(ref watermarkCreateRegionName, value);
        }

        private string _currentCreateName;
        public string CurrentCreateName
        {
            get => _currentCreateName;
            set => _currentCreateName = value;
        }

        private string _currentCreateModName;
        public string CurrentCreateModName
        {
            get => _currentCreateModName;
            set => _currentCreateModName = value;
        }

        private Visibility watermarkCreateVisible;
        public Visibility WatermarkCreateVisible
        {
            get => watermarkCreateVisible;
            set => SetProperty(ref watermarkCreateVisible, value);
        }

        private Visibility _createBaseVisible;
        public Visibility CreateBaseVisible
        {
            get => _createBaseVisible;
            set => SetProperty(ref _createBaseVisible, value);
        }

        private Visibility _editBaseVisible;
        public Visibility EditBaseVisible
        {
            get => _editBaseVisible;
            set => SetProperty(ref _editBaseVisible, value);
        }

        public DelegateCommand<object> ChangeCreateModCommand { get; set; }

        public DelegateCommand EnterTemplateListCommand { get; set; }

        public DelegateCommand SaveToTemplateListCommand { get; set; }

        public DelegateCommand SaveToCurrentTemplateListCommand { get; set; }

        public WatermarkCreateBaseContentViewModel(IRegionManager regionManager, IEventAggregator eventAggregator)
        {
            this.eventAggregator = eventAggregator;
            this.watermarkCreateRegion = regionManager;
            WatermarkCreateRegionName = Guid.NewGuid().ToString();
            ChangeCreateModCommand = new DelegateCommand<object>(ChangeCreateMod);
            EnterTemplateListCommand = new DelegateCommand(EnterTemplateList);
            SaveToTemplateListCommand = new DelegateCommand(SaveToTemplateList);

            SaveToCurrentTemplateListCommand = new DelegateCommand(SaveToCurrentTemplateList);

        }

        public void SaveToTemplateList()
        {
            if (CurrentCreateModName == "WatermarkCreateTextContent")
            {
                this.eventAggregator.GetEvent<SaveWatermarkTemplateEvent>().Publish(EnumTextOrFile.StatusText);
            }
            if (CurrentCreateModName == "WatermarkCreateFileContent")
            {
                this.eventAggregator.GetEvent<SaveWatermarkTemplateEvent>().Publish(EnumTextOrFile.StatusFile);
            }
        }

        public void SaveToCurrentTemplateList()
        {
            if (CurrentCreateModName == "WatermarkCreateTextContent")
            {
                eventAggregator.GetEvent<ConfirmEditWatermarkTemplateItemEvent>().Publish(EnumTextOrFile.StatusText);
            }
            if (CurrentCreateModName == "WatermarkCreateFileContent")
            {
                eventAggregator.GetEvent<ConfirmEditWatermarkTemplateItemEvent>().Publish(EnumTextOrFile.StatusFile);
            }
        }

        public void EnterTemplateList()
        {
            this.eventAggregator.GetEvent<EnterTemplateListOrCreateEvent>().Publish(EnumTemplateListOrCreate.StatusTemplate);
        }

        public void EnterSelectedCreateMod(string currentCreateName)
        {
            NavigationParameters param = new NavigationParameters();
            param.Add(ParameterNames.PDFViewer, PDFViewer);
            watermarkCreateRegion.RequestNavigate(WatermarkCreateRegionName, currentCreateName, param);
            watermarkCreateVisible = System.Windows.Visibility.Visible;
        }

        public void EditSelectedTemplateItem(string currentCreateName)
        {
            NavigationParameters param = new NavigationParameters();
            param.Add(ParameterNames.PDFViewer, PDFViewer);
            param.Add("WatermarkInfo", WatermarkInfo);
            watermarkCreateRegion.RequestNavigate(WatermarkCreateRegionName, currentCreateName, param);
            watermarkCreateVisible = System.Windows.Visibility.Visible;
        }

        public void ChangeCreateMod(object e)
        {
            var args = e as Button;
            if (args != null)
            {
                CurrentCreateName = args.Name;
                EnterSelectedCreateMod(CurrentCreateName);
                eventAggregator.GetEvent<SetCurrentCreateModEvent>().Publish(CurrentCreateName);
            }
        }

        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            navigationContext.Parameters.TryGetValue<CPDFViewer>(ParameterNames.PDFViewer, out PDFViewer);
            navigationContext.Parameters.TryGetValue<EnumTextOrFile>("CurrentTemplateListModName", out CurrentTemplateListMod);
            if (CurrentTemplateListMod == EnumTextOrFile.StatusText)
            {
                CurrentCreateModName = "WatermarkCreateTextContent";
            }
            else
            {
                CurrentCreateModName = "WatermarkCreateFileContent";
            }
            if (navigationContext.Parameters.TryGetValue<WatermarkInfo>("WatermarkInfo", out WatermarkInfo))
            {
                EditSelectedTemplateItem(CurrentCreateModName);
                EditBaseVisible = Visibility.Visible;
                CreateBaseVisible = Visibility.Collapsed;
            }
            else
            {
                EnterSelectedCreateMod(CurrentCreateModName);
                CreateBaseVisible = Visibility.Visible;
                EditBaseVisible = Visibility.Collapsed;
            }
            eventAggregator.GetEvent<SetCurrentCreateModEvent>().Publish(CurrentCreateModName);
        }
    }
}
